# This program is used to transform output into clear report
import sys
import os
import math

def main():
	#Get simulation arg!
	if len(sys.argv) != 5:
		print 'Usage:   python calresult <dirPath> <cnum> <times> <mode>'
		return
	dirPath = sys.argv[1]
	if dirPath.endswith('/') == False:
		dirPath += '/'
	nn = sys.argv[2]
	times = sys.argv[3]
	alphas = sys.argv[4]

	#read file
	dictionary = {'X': [], 'A': [], 'B': [], 'C': [], 'D': [], 'E': [], 'F': [], 'G': [], }

	#header
	print nn + '\t' + '\t'.join(alphas)
	
	#Data
	for time in range(int(times)):
		msg = '%4d\t' % time
		for alpha in alphas:
			result = parseFile(dirPath + str(nn) + '_' + alpha + '_' + str(time) + '.txt')
                        if (result>0) :
			    dictionary[alpha].append(result)
                        msg += '%3d\t' % result
		#print msg

	#Average
	msg = '%s\t' % 'AVG'
	avgList = avg(alphas, dictionary)
	for i in range(len(avgList)):
		msg += '%.2f\t' % avgList[i]
	#print msg


	if (int(times)<3):
		print msg
	else:
		#Standard Deviation
		msg = '%s\t' % 'STD'
		stdList = std(alphas, dictionary)
		for i in range(len(stdList)):
			msg += '%.2f\t' % stdList[i]
		#print msg

		#Confidence interval
		highList, lowList = CI(avgList, stdList, times)
		#print "CI:",highList, lowList
		msg = '%s\t' % 'CIH'
		for i in range(len(highList)):
			msg += '%.2f\t' % highList[i]
		#print msg
		msg = '%s\t' % 'CIL'
		for i in range(len(lowList)):
			msg += '%.2f\t' % lowList[i]
		#print msg

		#Check result
		dictionary =  check(dictionary, alphas, highList, lowList)
		#print dictionary

		#Re-Average
		msg = '%s\t' % 'RAVG'
		avgList = avg(alphas, dictionary)
		for i in range(len(avgList)):
			msg += '%.2f\t' % avgList[i]
		print msg

#Average
def avg(alphas, dictionary):
	result = []
	for alpha in alphas:
		result.append(float(sum(dictionary[alpha])) / len(dictionary[alpha]))
	return result

#Standard Deviation
def std(alphas, dictionary):
	result = []
	for alpha in alphas:
		avg = float(sum(dictionary[alpha])) / len(dictionary[alpha])
		s = 0
		for i in range(len(dictionary[alpha])):
			s += (dictionary[alpha][i]-avg)**2
		#delta = math.sqrt(s/(len(dictionary[alpha])-1))
		delta = math.sqrt(s/(len(dictionary[alpha])))
		result.append(delta)
	return result

#Confidence interval
def CI(avgList, stdList, times):
	highList = []
	lowList = []
	for i in range(len(avgList)):
		highList.append(avgList[i]+1.96*stdList[i]/math.sqrt(float(times)))
		lowList.append(avgList[i]-1.96*stdList[i]/math.sqrt(float(times)))
	return highList, lowList

#Check result
def check(dictionary, alphas, highList, lowList):
	for alpha, index in zip(alphas, range(len(highList))):
		removeList = []
		for i in range(len(dictionary[alpha])):
			if (dictionary[alpha][i]>highList[index]):
				removeList.append(dictionary[alpha][i])
			"""elif (dictionary[alpha][i]<lowList[index]):
				removeList.append(dictionary[alpha][i])"""
		for value in removeList:
			dictionary[alpha].remove(value)
	return dictionary

def parseFile(filePath):
	fd = open(filePath, 'r');
	lines = fd.readlines()
	max = -1
	min = 9999
	for line in lines:
		if line[0].isdigit() == True:
			tokens = line.split(' ')
			# print tokens[0]
			if float(tokens[0]) > max: max = float(tokens[0])
			if float(tokens[0]) < min: min = float(tokens[0])
	fd.close()
	return int(max - min)

main()
